-- @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
--   CORPORATE->TBL_CORPORATE_CUSTOM_DATA TBL_CORPORATE_CUSTOM_DATA TBL_CORPORATE_CUSTOM_DATA TBL_CORPORATE_CUSTOM_DATA  
-- @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
------------------------------------------------------------------------------------------
            CREATE OR REPLACE FUNCTION fn_get_all_corporate_custom_data() RETURNS SETOF corporate.tbl_corporate_custom_data AS $$
                DECLARE

                BEGIN
                    RETURN QUERY SELECT * FROM corporate.tbl_corporate_custom_data WHERE int_sys_action_id != -1;  
                END;
            $$ LANGUAGE plpgsql;
        

------------------------------------------------------------------------------------------
            CREATE OR REPLACE FUNCTION fn_get_all_corporate_custom_data_with_action_user() 
                            RETURNS TABLE (pk_corporate_custom_data_id BIGINT,

								vhr_sys_attribute_name VARCHAR,
								vhr_attribute_label VARCHAR,
								sin_category SMALLINT,
								sin_sub_category SMALLINT,
								sin_input_based_on SMALLINT,
								sin_data_type SMALLINT,
								sin_widget_type SMALLINT,
								art_select_items TEXT[],
								sin_use_in_set_rule SMALLINT,

								txt_remarks TEXT,
								sin_record_status SMALLINT,
								sin_order SMALLINT,


								int_sys_action_id INT,
								fk_created_user_id BIGINT,
								vhr_created_user VARCHAR,
								dtm_created TIMESTAMP WITH TIME ZONE,
								fk_last_modified_user_id BIGINT,
								vhr_last_modified_user VARCHAR,
								dtm_last_modified TIMESTAMP WITH TIME ZONE,
								fk_deleted_user_id BIGINT,
								vhr_deleted_user VARCHAR,
								dtm_deleted TIMESTAMP WITH TIME ZONE) AS $$
                DECLARE

                BEGIN
                    RETURN QUERY SELECT mt.pk_corporate_custom_data_id,

					mt.vhr_sys_attribute_name,
					mt.vhr_attribute_label,
					mt.sin_category,
					mt.sin_sub_category,
					mt.sin_input_based_on,
					mt.sin_data_type,
					mt.sin_widget_type,
					mt.art_select_items,
					mt.sin_use_in_set_rule,

					mt.txt_remarks,
					mt.sin_record_status,
					mt.sin_order,


					mt.int_sys_action_id,
					mt.fk_created_user_id,
					u1.vhr_login_name AS vhr_created_user,
					mt.dtm_created,
					mt.fk_last_modified_user_id,
					u2.vhr_login_name AS vhr_last_modified_user,
					mt.dtm_last_modified,
					mt.fk_deleted_user_id,
					u3.vhr_login_name AS vhr_deleted_user,
					mt.dtm_deleted
                        FROM corporate.tbl_corporate_custom_data mt 
                            LEFT JOIN authentication.tbl_user u1
                                ON mt.fk_created_user_id =  u1.pk_user_id
                            LEFT JOIN authentication.tbl_user u2
                                ON mt.fk_last_modified_user_id =  u2.pk_user_id
                            LEFT JOIN authentication.tbl_user u3
                                ON mt.fk_deleted_user_id =  u3.pk_user_id
                        WHERE mt.int_sys_action_id != -1  
                        ORDER BY sin_order, vhr_sys_attribute_name;
                END;
            $$ LANGUAGE plpgsql;
        

------------------------------------------------------------------------------------------

            CREATE OR REPLACE PROCEDURE sp_insert_or_update_corporate_custom_data(jsnCorporateCustomData JSONB, jsnSqlRes OUT JSON) AS $$
                DECLARE
                    objSpOutRes TYP_SP_OUT_RES;
                    objFnCheckRes TYP_FN_CHECK_RES;
                    dtmCurrent TIMESTAMP WITH TIME ZONE;
                BEGIN
                    SELECT NOW() INTO dtmCurrent;

                    -- // Duplication Checking in DB FUCTION(Insert and Update)
                    SELECT * FROM  fn_check_data_uniqueness('corporate.tbl_corporate_custom_data'::VARCHAR, ''::VARCHAR, ::VARCHAR, 'pk_corporate_custom_data_id'::VARCHAR, CAST(jsnCorporateCustomData->>'binCorporateCustomDataId' AS BIGINT)::BIGINT) INTO objFnCheckRes;
                    IF objFnCheckRes.sin_sql_check_status = -1::SMALLINT THEN
                        objSpOutRes.sinSqlCustomStatus := -1::SMALLINT;
                        objSpOutRes.strSqlCustomStatusCode := 'ERROR';
                        objSpOutRes.strSqlCustomMessage := objFnCheckRes.vhr_sql_check_message;

                        objSpOutRes.strSqlCustomErrorType := 'DATABASE_VALIDATION_ERROR';

                        SELECT * FROM  fn_convert_type_out_object_to_out_json(objSpOutRes) INTO jsnSqlRes;
                        RAISE NOTICE 'jsnSqlRes: %', jsnSqlRes;
                        RETURN;
                    END IF;

                    IF CAST(jsnCorporateCustomData->>'binCorporateCustomDataId' AS BIGINT)::BIGINT = 0::BIGINT THEN
                        -- // Count Exced or other checking License related in DB FUCTION(Insert)
                        

                        --jsnCorporateCustomData.intSysActionId = 0;
                        --jsnCorporateCustomData.binCreatedUserId = jsnCorporateCustomData.linLoginUserId;
                        --jsnCorporateCustomData.dtmCreated = jsnCorporateCustomData.dtmCurrentDateTime; --SELECT NOW();

                        INSERT INTO corporate.tbl_corporate_custom_data (vhr_sys_attribute_name,
						vhr_attribute_label,
						sin_category,
						sin_sub_category,
						sin_input_based_on,
						sin_data_type,
						sin_widget_type,
						art_select_items,
						sin_use_in_set_rule,

						txt_remarks,
						sin_record_status,
						sin_order,


						int_sys_action_id,
						fk_created_user_id,
						dtm_created)
                            VALUES(jsnCorporateCustomData->>'strSysAttributeName',
						jsnCorporateCustomData->>'strAttributeLabel',
						CAST(jsnCorporateCustomData->>'sinCategory' AS SMALLINT),
						CAST(jsnCorporateCustomData->>'sinSubCategory' AS SMALLINT),
						CAST(jsnCorporateCustomData->>'sinInputBasedOn' AS SMALLINT),
						CAST(jsnCorporateCustomData->>'sinDataType' AS SMALLINT),
						CAST(jsnCorporateCustomData->>'sinWidgetType' AS SMALLINT),
						STRING_TO_ARRAY(jsnCorporateCustomData->>'strDbArtSelectItems', ',')::TEXT[],
						CAST(jsnCorporateCustomData->>'sinUseInSetRule' AS SMALLINT),

						CAST(jsnCorporateCustomData->>'strRemarks' AS TEXT),
						CAST(jsnCorporateCustomData->>'sinRecordStatus' AS SMALLINT),
						CAST(jsnCorporateCustomData->>'sinOrder' AS SMALLINT),


						0,
						CAST(jsnCorporateCustomData->>'binCreatedUserId' AS BIGINT),
						dtmCurrent) RETURNING pk_corporate_custom_data_id INTO objSpOutRes.binSqlPk;
                    ELSE
                        -- // UPDATE Check already update another usersin DB FUCTION(Update and Delete)
                        SELECT * FROM  fn_check_already_update_or_delete_another_user('corporate.tbl_corporate_custom_data'::VARCHAR, 'pk_corporate_custom_data_id'::VARCHAR, CAST(jsnCorporateCustomData->>'binCorporateCustomDataId' AS BIGINT)::BIGINT, CAST(jsnCorporateCustomData->>'intSysActionId' AS INT)::INT) INTO objFnCheckRes;
                        IF objFnCheckRes.sin_sql_check_status = -1::SMALLINT THEN
                            objSpOutRes.sinSqlCustomStatus := -1::SMALLINT;
                            objSpOutRes.strSqlCustomStatusCode := 'ERROR';
                            objSpOutRes.strSqlCustomMessage := objFnCheckRes.vhr_sql_check_message;

                            objSpOutRes.strSqlCustomErrorType := 'DATABASE_VALIDATION_ERROR';

                            SELECT * FROM  fn_convert_type_out_object_to_out_json(objSpOutRes) INTO jsnSqlRes;
                            RAISE NOTICE 'jsnSqlRes: %', jsnSqlRes;
                            RETURN;
                        END IF;

                        --jsnCorporateCustomData.intSysActionId = jsnCorporateCustomData.intSysActionId + 1;
                        --jsnCorporateCustomData.binLastModifiedUserId = jsnCorporateCustomData.linLoginUserId;
                        --jsnCorporateCustomData.dtmLastModified = jsnCorporateCustomData.dtmCurrentDateTime; --SELECT NOW();

                        UPDATE corporate.tbl_corporate_custom_data SET vhr_sys_attribute_name = jsnCorporateCustomData->>'strSysAttributeName',
						vhr_attribute_label = jsnCorporateCustomData->>'strAttributeLabel',
						sin_category = CAST(jsnCorporateCustomData->>'sinCategory' AS SMALLINT),
						sin_sub_category = CAST(jsnCorporateCustomData->>'sinSubCategory' AS SMALLINT),
						sin_input_based_on = CAST(jsnCorporateCustomData->>'sinInputBasedOn' AS SMALLINT),
						sin_data_type = CAST(jsnCorporateCustomData->>'sinDataType' AS SMALLINT),
						sin_widget_type = CAST(jsnCorporateCustomData->>'sinWidgetType' AS SMALLINT),
						art_select_items = STRING_TO_ARRAY(jsnCorporateCustomData->>'strDbArtSelectItems', ',')::TEXT[],
						sin_use_in_set_rule = CAST(jsnCorporateCustomData->>'sinUseInSetRule' AS SMALLINT),

						txt_remarks = jsnCorporateCustomData->>'strRemarks',
						sin_record_status = CAST(jsnCorporateCustomData->>'sinRecordStatus' AS SMALLINT),
						sin_order = CAST(jsnCorporateCustomData->>'sinOrder' AS SMALLINT),


						int_sys_action_id = int_sys_action_id + 1,
						fk_last_modified_user_id = CAST(jsnCorporateCustomData->>'binLastModifiedUserId' AS BIGINT),
						dtm_last_modified = dtmCurrent 
                        WHERE pk_corporate_custom_data_id = CAST(jsnCorporateCustomData->>'binCorporateCustomDataId' AS BIGINT);
                    END IF;

                    --COMMIT;
                    objSpOutRes.sinSqlCustomStatus := 1::SMALLINT;
                    objSpOutRes.strSqlCustomStatusCode := 'SUCCESS';
                    objSpOutRes.strSqlCustomMessage := 'CorporateCustomData Saved';
                    objSpOutRes.dtmCurrent = dtmCurrent;
                    
                    SELECT * FROM  fn_convert_type_out_object_to_out_json(objSpOutRes) INTO jsnSqlRes;
                    RAISE NOTICE 'jsnSqlRes: %', jsnSqlRes;
                    RETURN;
                EXCEPTION WHEN OTHERS THEN
                    ROLLBACK;
                    objSpOutRes.sinSqlCustomStatus := -1::SMALLINT;
                    objSpOutRes.strSqlCustomStatusCode := 'ERROR';
                    objSpOutRes.strSqlCustomMessage := 'CorporateCustomData Save Failed';

                    objSpOutRes.strSqlCustomErrorType := 'DATABASE_SYSTEM_ERROR';

                    GET STACKED DIAGNOSTICS 
				        objSpOutRes.strSqlSysMessage := MESSAGE_TEXT,
                    	objSpOutRes.strSqlSysExceptionDetail := PG_EXCEPTION_DETAIL,
                    	objSpOutRes.strSqlSysExceptionHint := PG_EXCEPTION_CONTEXT;
                
                    SELECT * FROM  fn_convert_type_out_object_to_out_json(objSpOutRes) INTO jsnSqlRes;
                    RAISE EXCEPTION 'jsnSqlRes: %', jsnSqlRes;
                    RETURN;
                END;
            $$ LANGUAGE plpgsql;
        

------------------------------------------------------------------------------------------

            CREATE OR REPLACE PROCEDURE sp_delete_corporate_custom_data(binCorporateCustomDataId BIGINT, intSysActionId INT, binDeletedUserId BIGINT, jsnSqlRes OUT JSON) AS $$
                DECLARE
                    objSpOutRes TYP_SP_OUT_RES;
                    objFnCheckRes TYP_FN_CHECK_RES;
                    dtmCurrent TIMESTAMP WITH TIME ZONE;
                BEGIN
                    SELECT NOW() INTO dtmCurrent;

                    -- // FOREIGN KEY REFERENCES Checking in DB FUCTION(Delete)
                    SELECT * FROM  fn_check_foreignkey_referance('tbl_corporate_custom_data'::VARCHAR, 'pk_corporate_custom_data_id'::VARCHAR, binCorporateCustomDataId::BIGINT, '{}'::VARCHAR ARRAY) INTO objFnCheckRes;
                    IF objFnCheckRes.sin_sql_check_status = -1::SMALLINT THEN
                        objSpOutRes.sinSqlCustomStatus := -1::SMALLINT;
                        objSpOutRes.strSqlCustomStatusCode := 'ERROR';
                        objSpOutRes.strSqlCustomMessage := objFnCheckRes.vhr_sql_check_message;

                        objSpOutRes.strSqlCustomErrorType := 'DATABASE_VALIDATION_ERROR';

                        SELECT * FROM  fn_convert_type_out_object_to_out_json(objSpOutRes) INTO jsnSqlRes;
                        RAISE NOTICE 'jsnSqlRes: %', jsnSqlRes;
                        RETURN;
                    END IF;

                    -- // UPDATE Check already update another users in DB FUCTION(Update and Delete)
                    SELECT * FROM  fn_check_already_update_or_delete_another_user('corporate.tbl_corporate_custom_data'::VARCHAR, 'pk_corporate_custom_data_id'::VARCHAR, CAST(binCorporateCustomDataId AS BIGINT)::BIGINT, intSysActionId::INT) INTO objFnCheckRes;
                    IF objFnCheckRes.sin_sql_check_status = -1::SMALLINT THEN
                        objSpOutRes.sinSqlCustomStatus := -1::SMALLINT;
                        objSpOutRes.strSqlCustomStatusCode := 'ERROR';
                        objSpOutRes.strSqlCustomMessage := objFnCheckRes.vhr_sql_check_message;

                        objSpOutRes.strSqlCustomErrorType := 'DATABASE_VALIDATION_ERROR';

                        SELECT * FROM  fn_convert_type_out_object_to_out_json(objSpOutRes) INTO jsnSqlRes;
                        RAISE NOTICE 'jsnSqlRes: %', jsnSqlRes;
                        RETURN;
                    END IF;

                    --dtmDeleted = SELECT NOW();
                    UPDATE corporate.tbl_corporate_custom_data SET int_sys_action_id = -1::INT,
                                fk_deleted_user_id = binDeletedUserId,
                                dtm_deleted = dtmCurrent
                        WHERE pk_corporate_custom_data_id = binCorporateCustomDataId;

                    --COMMIT;
                    objSpOutRes.sinSqlCustomStatus := 1::SMALLINT;
                    objSpOutRes.strSqlCustomStatusCode := 'SUCCESS';
                    objSpOutRes.strSqlCustomMessage := 'CorporateCustomData Deleted';
                    objSpOutRes.dtmCurrent = dtmCurrent;

                    SELECT * FROM fn_convert_type_out_object_to_out_json(objSpOutRes) INTO jsnSqlRes;
                    RAISE NOTICE 'jsnSqlRes: %', jsnSqlRes;
                    RETURN;
                EXCEPTION WHEN OTHERS THEN
                    ROLLBACK;
                    objSpOutRes.sinSqlCustomStatus := -1::SMALLINT;
                    objSpOutRes.strSqlCustomStatusCode := 'ERROR';
                    objSpOutRes.strSqlCustomMessage := 'CorporateCustomData Delete Failed';

                    objSpOutRes.strSqlCustomErrorType := 'DATABASE_SYSTEM_ERROR';

                    GET STACKED DIAGNOSTICS 
				        objSpOutRes.strSqlSysMessage := MESSAGE_TEXT,
                    	objSpOutRes.strSqlSysExceptionDetail := PG_EXCEPTION_DETAIL,
                    	objSpOutRes.strSqlSysExceptionHint := PG_EXCEPTION_CONTEXT;

                    SELECT * FROM fn_convert_type_out_object_to_out_json(objSpOutRes) INTO jsnSqlRes;
                    RAISE EXCEPTION 'jsnSqlRes: %', jsnSqlRes;
                    RETURN;
                END;
            $$ LANGUAGE plpgsql;
        

